package sma.ep2.beans.fachada;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import sma.ep2.beans.aluno.AlunoLocal;
import sma.ep2.beans.aluno.AlunoLocalHome;
import sma.ep2.beans.disciplina.DisciplinaLocal;
import sma.ep2.beans.disciplina.DisciplinaLocalHome;
import sma.ep2.beans.fachada.vo.AlunoValueObject;
import sma.ep2.beans.fachada.vo.DisciplinaValueObject;

/**
 * Bean de sesso sem estado que prov acesso aos entity beans (<code>AlunoBean</code> e
 * <code>DisciplinaBean</code>).
 */
public class FachadaBean implements SessionBean
{

   private AlunoLocalHome alunoLocalHome;

   private DisciplinaLocalHome disciplinaLocalHome;

   public void criaAluno(Integer numeroUsp, String nome, String curso, String email)
   {
      try
      {
         AlunoLocal aluno = alunoLocalHome.create(numeroUsp, nome, curso, email);
      }
      catch (CreateException e)
      {
         throw new EJBException(e);
      }
   }

   public void criaDisciplina(String sigla, String nome, String professor, String horario)
   {
      try
      {
         DisciplinaLocal disciplina = disciplinaLocalHome.create(sigla, nome, professor, horario);
      }
      catch (CreateException e)
      {
         throw new EJBException(e);
      }

   }

   public void removeAluno(Integer numeroUsp) throws FinderException
   {
      AlunoLocal aluno = alunoLocalHome.findByPrimaryKey(numeroUsp);
      try
      {
         aluno.remove();
      }
      catch (RemoveException e)
      {
         throw new EJBException(e);
      }
   }

   public void removeDisciplina(String sigla) throws FinderException
   {
      DisciplinaLocal disciplina = disciplinaLocalHome.findByPrimaryKey(sigla);
      try
      {
         disciplina.remove();
      }
      catch (RemoveException e)
      {
         throw new EJBException(e);
      }
   }

   public void efetuaMatricula(Integer numeroUspDoAluno, String siglaDaDisciplina)
         throws FinderException
   {
      AlunoLocal aluno = alunoLocalHome.findByPrimaryKey(numeroUspDoAluno);
      DisciplinaLocal disciplina = disciplinaLocalHome.findByPrimaryKey(siglaDaDisciplina);
      aluno.getDisciplinas().add(disciplina);
      disciplina.getAlunos().add(aluno);
   }

   public AlunoValueObject obtemAluno(Integer numeroUsp) throws FinderException
   {
      AlunoLocal bean = alunoLocalHome.findByPrimaryKey(numeroUsp);
      return toAlunoValueObject(bean);
   }

   public DisciplinaValueObject obtemDisciplina(String sigla) throws FinderException
   {
      DisciplinaLocal bean = disciplinaLocalHome.findByPrimaryKey(sigla);
      return toDisciplinaValueObject(bean);
   }

   public List obtemTodosOsAlunos() throws FinderException
   {
      ArrayList alunos = new ArrayList();
      Collection beans = alunoLocalHome.findAllAlunos();
      Iterator it = beans.iterator();
      while (it.hasNext())
      {
         AlunoLocal bean = (AlunoLocal) it.next();
         AlunoValueObject alunoVO = toAlunoValueObject(bean);
         alunos.add(alunoVO);
      }

      return alunos;
   }

   public List obtemTodasAsDisciplinas() throws FinderException
   {
      ArrayList disciplinas = new ArrayList();
      Collection beans = disciplinaLocalHome.findAllDisciplinas();
      Iterator it = beans.iterator();
      while (it.hasNext())
      {
         DisciplinaLocal bean = (DisciplinaLocal) it.next();
         DisciplinaValueObject disciplinaVO = toDisciplinaValueObject(bean);
         disciplinas.add(disciplinaVO);
      }

      return disciplinas;
   }

   // Mtodos de callback ---------------------------------------------------
   public void ejbCreate()
   {
      try
      {
         Context context = new InitialContext();
         Object refAlunoLocalHome = context.lookup("java:comp/env/ejb/AlunoLocal");
         alunoLocalHome = (AlunoLocalHome) PortableRemoteObject.narrow(refAlunoLocalHome,
               AlunoLocalHome.class);
         Object refDisciplinaLocalHome = context.lookup("java:comp/env/ejb/DisciplinaLocal");
         disciplinaLocalHome = (DisciplinaLocalHome) PortableRemoteObject.narrow(
               refDisciplinaLocalHome, DisciplinaLocalHome.class);

      }
      catch (NamingException e)
      {
         throw new EJBException(e);
      }
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbRemove()
   {
   }

   public void setSessionContext(SessionContext sessionContext)
   {
   }

   // Private ---------------------------------------------------------------
   private AlunoValueObject toAlunoValueObject(AlunoLocal bean)
   {
      Collection disciplinas = bean.getDisciplinas();
      Iterator it = disciplinas.iterator();
      ArrayList disciplinasDoAluno = new ArrayList();
      while (it.hasNext())
      {
         DisciplinaLocal disciplina = (DisciplinaLocal) it.next();
         DisciplinaValueObject disciplinaVO = new DisciplinaValueObject(disciplina.getSigla(),
               disciplina.getNome(), disciplina.getProfessor(), disciplina.getHorario(), null);
         disciplinasDoAluno.add(disciplinaVO);
      }
      return new AlunoValueObject(bean.getNumeroUsp(), bean.getNome(), bean.getCurso(), bean
            .getEmail(), disciplinasDoAluno);
   }

   private DisciplinaValueObject toDisciplinaValueObject(DisciplinaLocal bean)
   {
      Collection alunos = bean.getAlunos();
      Iterator it = alunos.iterator();
      ArrayList alunosDaDisciplina = new ArrayList();
      while (it.hasNext())
      {
         AlunoLocal aluno = (AlunoLocal) it.next();
         AlunoValueObject alunoVO = new AlunoValueObject(aluno.getNumeroUsp(), aluno.getNome(),
               aluno.getCurso(), aluno.getEmail(), null);
         alunosDaDisciplina.add(alunoVO);
      }
      return new DisciplinaValueObject(bean.getSigla(), bean.getNome(), bean.getProfessor(), bean
            .getHorario(), alunosDaDisciplina);
   }

}
